@@ -37,7 +37,8 @@ from ..docs import OpenAPIResponse, swagger_auto_schema
 from ..mixins import AccountMixin
 from ..models import PortfolioDoubleOptIn
 from .serializers import (NoModelSerializer, PortfolioOptInSerializer,
-    PortfolioGrantCreateSerializer, PortfolioRequestCreateSerializer)
+    PortfolioOptInUpdateSerializer, PortfolioGrantCreateSerializer,
+    PortfolioRequestCreateSerializer)
 from ..filters import (DateRangeFilter, DoubleOptInStateFilter, OrderingFilter,
      SearchFilter)
 from ..utils import datetime_or_now, get_account_model
@@ -197,16 +198,22 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
         .. code-block:: json
 
             {
-               "grantee": {
-                 "slug": "energy-utility",
-                 "full_name": "Energy Utility"
-               }
+               "count": 1,
+               "results": [{
+                 "grantee": "water-utility",
+                 "account": "supplier-1",
+                 "campaign": "sustainability",
+                 "ends_at": "2022-01-01T00:00:00Z",
+                 "state": "grant-initiated",
+                 "api_accept": "/api/water-utility/portfolios/grants\
+/0000000000000000000000000000000000000003/"
+               }]
             }
         """
+        #pylint:disable=too-many-locals
         serializer = self.get_serializer(data=request.data)
         serializer.is_valid(raise_exception=True)
 
-        #pylint:disable=too-many-locals
         created_at = datetime_or_now()
         # If we don't make a copy, we will get an exception "Got KeyError
         # when attempting to get a value for field `slug`" later on in
@@ -221,8 +228,9 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
         account_model = get_account_model()
         lookups = {self.lookup_field: grantee_data.pop('slug')}
         invalid_fields = []
-        model_fields = set([
-            field.name for field in account_model._meta.get_fields()])
+        # set comprehension py2.7+ syntax
+        model_fields = {
+            field.name for field in account_model._meta.get_fields()}
         for field_name in six.iterkeys(grantee_data):
             if field_name not in model_fields:
                 invalid_fields += [field_name]
@@ -290,8 +298,11 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
         for portfolio in requests_accepted:
             signals.portfolio_request_accepted.send(sender=__name__,
                 portfolio=portfolio, request=self.request)
+
+        message = serializer.validated_data.get('message')
         signals.portfolios_grant_initiated.send(sender=__name__,
-            portfolios=portfolios, invitee=grantee_data, request=self.request)
+            portfolios=portfolios, invitee=grantee_data, message=message,
+            request=self.request)
 
         results = []
         serializer_class = PortfolioOptInSerializer
@@ -522,47 +533,21 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
                         grantee=self.account,
                         account=account,
                         campaign=campaign,
+                    verification_key=PortfolioDoubleOptIn.generate_key(account),
                         **defaults)
                 requests_initiated += [(portfolio, account)]
 
+        message = serializer.validated_data.get('message')
         for request in requests_initiated:
             portfolio = request[0]
             account = request[1]
-            signals.portfolio_request_initiated.send(sender=__name__,
-                portfolio=portfolio, invitee=account, request=self.request)
+            signals.portfolios_request_initiated.send(sender=__name__,
+                portfolios=[portfolio], invitee=account, message=message,
+                request=self.request)
 
 
 class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
-    """
-    Accepts a portfolio request
 
-    A `grantee` has made a request to *account*'s portfolio. The *account*
-    accepts the request, making the *account*'s answers up-to-date `ends_at`
-    available to `grantee`.
-
-    Note that *account* is the actual *account* we are looking to access data
-    from here, while the *account* parameter is the `grantee` when calling
-    `POST /api/{account}/requests`.
-
-    **Tags**: portfolios
-
-    **Examples**
-
-    .. code-block:: http
-
-        POST /api/supplier-1/portfolios/requests/0123456789abcef HTTP/1.1
-
-    .. code-block:: json
-
-        {}
-
-    responds
-
-    .. code-block:: json
-
-        {}
-    """
-#    lookup_url_kwarg = 'verification_key'
     lookup_field = 'verification_key'
     serializer_class = NoModelSerializer
 
@@ -570,14 +555,48 @@ class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
         return PortfolioDoubleOptIn.objects.filter(account=self.account)
 
     def post(self, request, *args, **kwargs):
+        """
+        Accepts a portfolio request
+
+        A `grantee` has made a request to *account*'s portfolio. The *account*
+        accepts the request, making the *account*'s answers up-to-date `ends_at`
+        available to `grantee`.
+
+        Note that *account* is the actual *account* we are looking to access data
+        from here, while the *account* parameter is the `grantee` when calling
+        `POST /api/{account}/requests`.
+
+        **Tags**: portfolios
+
+        **Examples**
+
+        .. code-block:: http
+
+            POST /api/supplier-1/portfolios/requests/0123456789abcef HTTP/1.1
+
+        .. code-block:: json
+
+            {}
+
+        responds
+
+        .. code-block:: json
+
+            {}
+        """
         #pylint:disable=unused-argument
         instance = self.get_object()
         self.perform_create(instance)
         return HttpResponse(status=status.HTTP_201_CREATED)
 
+    def perform_create(self, instance):
+        instance.request_accepted()
+        signals.portfolio_request_accepted.send(sender=__name__,
+            portfolio=instance, request=self.request)
+
     def delete(self, request, *args, **kwargs):
         """
-        Denies a portfolio request
+        Removes or denies a portfolio request
 
         **Tags**: portfolios
 
@@ -588,15 +607,93 @@ class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
             DELETE /api/supplier-1/portfolios/requests/0123456789abcef\
  HTTP/1.1
         """
-        return self.destroy(request, *args, **kwargs)
+        filter_args = {self.lookup_field: self.kwargs.get(self.lookup_field)}
+        try:
+            instance = self.get_queryset().get(**filter_args)
+            instance.state = PortfolioDoubleOptIn.OPTIN_REQUEST_DENIED
+            instance.save()
+            signals.portfolio_request_denied.send(sender=__name__,
+                portfolio=instance, request=self.request)
+            return HttpResponse(status=status.HTTP_204_NO_CONTENT)
+        except PortfolioDoubleOptIn.DoesNotExist:
+            pass
 
-    def perform_create(self, instance):
-        instance.request_accepted()
-        signals.portfolio_request_accepted.send(sender=__name__,
-            portfolio=instance, request=self.request)
+        # We cannot find the grant to the grantee, so let's look
+        # if the grant was removed by any chance.
+        instance = generics.get_object_or_404(
+            PortfolioDoubleOptIn.objects.filter(grantee=self.account),
+            **filter_args)
+        instance.delete()
+        return HttpResponse(status=status.HTTP_204_NO_CONTENT)
 
-    def perform_destroy(self, instance):
-        instance.state = PortfolioDoubleOptIn.OPTIN_REQUEST_DENIED
-        instance.save()
-        signals.portfolio_request_denied.send(sender=__name__,
-            portfolio=instance, request=self.request)
+
+class PortfoliosUpdateAPIView(AccountMixin, generics.UpdateAPIView):
+    """
+    Update extra information in a portfolio request/grant
+
+    The requestor/grantor uses this API call to add metadata about
+    the request/grant.
+
+    **Tags**: portfolios
+
+    **Examples**
+
+    .. code-block:: http
+
+        PUT /api/energy-utility/portfolios/requests/metadata/supplier-1 HTTP/1.1
+
+    .. code-block:: json
+
+        {
+          "extra": {"tags": "tier1"}
+        }
+
+    responds
+
+    .. code-block:: json
+
+        {
+          "grantee": "energy-utility",
+          "account": "supplier-1",
+          "campaign": "sustainability",
+          "ends_at": "2022-01-01T00:00:00Z",
+          "state": "request-denied",
+          "api_accept": null,
+          "extra": {"tags": "tier1"}
+        }
+    """
+    target_url_kwarg = 'target'
+    serializer_class = PortfolioOptInSerializer
+
+    def get_serializer_class(self):
+        if self.request.method.lower() in ('put', 'patch'):
+            return PortfolioOptInUpdateSerializer
+        return super(PortfoliosUpdateAPIView, self).get_serializer_class()
+
+    @swagger_auto_schema(responses={
+      200: OpenAPIResponse("Update successful", PortfolioOptInSerializer,
+      many=False)})
+    def put(self, request, *args, **kwargs):
+        return super(PortfoliosUpdateAPIView, self).put(
+            request, *args, **kwargs)
+
+    @swagger_auto_schema(responses={
+      200: OpenAPIResponse("Update successful", PortfolioOptInSerializer,
+      many=False)})
+    def patch(self, request, *args, **kwargs):
+        return super(PortfoliosUpdateAPIView, self).patch(
+            request, *args, **kwargs)
+
+    def update(self, request, *args, **kwargs):
+        partial = kwargs.pop('partial', False)
+        serializer = self.get_serializer(data=request.data, partial=partial)
+        serializer.is_valid(raise_exception=True)
+
+        optins = PortfolioDoubleOptIn.objects.requested(self.account).filter(
+            account__slug=self.kwargs.get(self.target_url_kwarg)
+        )
+
+        optins.update(extra=serializer.validated_data.get('extra'))
+
+        last_optin = optins.order_by('-created_at').first()
+        return HttpResponse(self.get_serializer(instance=last_optin).data)
