@@ -204,8 +204,7 @@ class QuestionCreateSerializer(serializers.ModelSerializer):
 
     class Meta:
         model = get_question_model()
-        fields = ('title', 'text', 'default_unit', 'correct_answer',
-            'extra')
+        fields = ('title', 'text', 'default_unit', 'extra')
 
 
 class QuestionDetailSerializer(QuestionCreateSerializer):
@@ -482,6 +481,8 @@ class TableSerializer(NoModelSerializer):
     printable_name = serializers.CharField(
         required=False, read_only=True,
         help_text=_("Name that can be safely used for display in HTML pages"))
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
     values = serializers.ListField(
         child=KeyValueTuple(),
         help_text="Datapoints in the serie")
@@ -500,6 +501,18 @@ class MetricsSerializer(NoModelSerializer):
     results = TableSerializer(many=True)
 
 
+class CompareQuestionSerializer(CampaignQuestionSerializer):
+    """
+    Serializer for a question in the `results` field of a Compare API call.
+    """
+    values = serializers.ListField(
+        child=serializers.ListField(child=AnswerSerializer()), required=False)
+
+    class Meta(CampaignQuestionSerializer.Meta):
+        fields = CampaignQuestionSerializer.Meta.fields + ('values',)
+
+
+
 class InviteeSerializer(NoModelSerializer):
 
     slug = serializers.SlugField()
