@@ -24,6 +24,7 @@
 
 #pylint: disable=invalid-name,no-name-in-module,unused-import,import-error
 #pylint: disable=no-name-in-module,unused-import
+import re
 from functools import WRAPPER_ASSIGNMENTS
 import six
 
@@ -60,10 +61,20 @@ except ModuleNotFoundError: #pylint:disable=undefined-variable,bad-except-order
     from django.core.urlresolvers import NoReverseMatch, reverse, reverse_lazy
 
 try:
-    from django.urls import include, re_path
+    from django.urls import include, path, re_path
 except ImportError: # <= Django 2.0, Python<3.6
     from django.conf.urls import include, url as re_path
 
+    def path(route, view, kwargs=None, name=None):
+        re_route = re.sub(
+                   r'<int:([a-z\_]+)>',
+                   r'(?P<\1>[0-9]+)',
+            re.sub(r'<path:([a-z\_]+)>',
+                   r'(?P<\1>([a-zA-Z0-9\-]+/)*[a-zA-Z0-9\-]+)',
+            re.sub(r'<slug:([a-z\_]+)>',
+                   r'(?P<\1>[a-zA-Z0-9_\-\+\.]+)',
+                   route)))
+        return re_path(re_route, view, kwargs=kwargs, name=name)
 
 try:
     from django.utils.encoding import python_2_unicode_compatible
